####################################################################;
# FUNCTIONS FOR PANEL ESTIMATION WITH INDIVIDUAL AND TIME EFFECTS ;
#####################################################################;



###################
# Bias Correction #
###################
bias.cor.beta <- function(model.spec, data.local, lin, type){
  n.K <- length(all.vars(model.spec))-3;
  n.I <- length(unique(data.local[,all.vars(model.spec)[n.K+2]]));
  n.J <- length(unique(data.local[,all.vars(model.spec)[n.K+3]]));
  n.obs <- NROW(data.local);
  
  X.name <- all.vars(model.spec)[2:(n.K+1)];
  i.name <- all.vars(model.spec)[n.K+2];
  j.name <- all.vars(model.spec)[n.K+3];
  
  X <- as.matrix(data.local[,X.name]);
  id <- data.local[,i.name];
  jd <- data.local[,j.name];
  
  if(type == "logit"){
    lambda.1 <- dlogis(lin);
    lambda.2 <- (1-2*plogis(lin))*lambda.1;
  };  
  
  X.hat <- lm(as.formula(paste("X~factor(",i.name,")+factor(",j.name,")",spe="")), weights=lambda.1, data=data.local)$res;
  if(length(X.hat)==0){return(0)};
  
  W <- (t(c(lambda.1)*X.hat) %*% X.hat)/n.obs;
  
  numerator <- c(lambda.2)*X.hat;
  temp.data <- data.frame(numerator,lambda.1,id,jd);
  
  B <- -1/2 * colMeans(aggregate(numerator~id,data=temp.data,sum,na.action=na.omit)[,-1]/
                      aggregate(lambda.1~id,data=temp.data,sum,na.action=na.omit)[,-1],na.rm=T) %*% solve(W);
  D <- -1/2 * colMeans(aggregate(numerator~jd,data=temp.data,sum,na.action=na.omit)[,-1]/
                      aggregate(lambda.1~jd,data=temp.data,sum,na.action=na.omit)[,-1],na.rm=T) %*% solve(W);
  
  return(B*n.I/n.obs + D*n.J/n.obs);
};


# Output of bias.cor.F.logit is the corrected F, not the correction part #

bias.cor.F.logit <- function(y, model.spec, data.local, lin, theta, ttype, t, treated){
  n.K <- length(all.vars(model.spec))-3;
  n.I <- length(unique(data.local[,all.vars(model.spec)[n.K+2]]));
  n.J <- length(unique(data.local[,all.vars(model.spec)[n.K+3]]));
  n.obs <- NROW(data.local);
  
  Y.name <- all.vars(model.spec)[1];
  X.name <- all.vars(model.spec)[2:(n.K+1)];
  i.name <- all.vars(model.spec)[n.K+2];
  j.name <- all.vars(model.spec)[n.K+3];
  
  X <- as.matrix(data.local[,X.name]);
  id <- data.local[,i.name];
  jd <- data.local[,j.name];
  
  lambda.1 <- dlogis(lin);
  lambda.2 <- (1-2*plogis(lin))*lambda.1;
    
  beta <- theta[1:n.K];
  beta.corrected <- beta - bias.cor.beta(model.spec, data.local, lin, "logit");
    
  ycurrent <- 1*(data.local[,Y.name]<=y); # I(Y<=y);
  om <- X %*% as.vector(beta.corrected);
  # Logit 1(Y<=y) on factor(id)+factor(jd) #
  lin.uncorrected <- glm(ycurrent~factor(id)+factor(jd), family=binomial(link="logit"), offset=om, na.action=na.omit)$linear.predictors;
  
  
  if(ttype=="log2" & treated==0){return(sum(plogis(lin.uncorrected),na.rm=T)/n.obs)};
  if(all(lambda.1==0)){return(sum(plogis(lin.uncorrected),na.rm=T)/n.obs)};
  
  
  if(ttype == "log2"){
    lin.counter <- lin + log(2)*theta[t];
    lin.uncorrected.counter <- lin.uncorrected + log(2)*beta.corrected[t];
  }else if(treated == 0){
    lin.counter <- lin - X[,t]*theta[t];
    lin.uncorrected.counter <- lin.uncorrected - X[,t]*beta.corrected[t];
  }else{
    lin.counter <- lin + (1-X[,t])*theta[t];
    lin.uncorrected.counter <- lin.uncorrected + (1-X[,t])*beta.corrected[t];
  };

  F.uncorrected <- sum(plogis(lin.uncorrected.counter), na.rm=T)/n.obs;
  
  lambda.1.counter <- dlogis(lin.counter);
  lambda.2.counter <- (1-2*plogis(lin.counter))*lambda.1.counter;
  
  
  Psi.hat <- lm(formula=lambda.1.counter/lambda.1~factor(id)+factor(jd), weights=lambda.1)$fitted.values;
  
  numerator <- lambda.2.counter - lambda.2*Psi.hat;
  temp.data <- data.frame(numerator,lambda.1,id,jd);
  
  B <- 1/2 * sum(aggregate(numerator~id,data=temp.data,sum,na.action=na.omit)[,-1]/aggregate(lambda.1~id,data=temp.data,sum,na.action=na.omit)[,-1],
                 na.em=T)/n.I;
  D <- 1/2 * sum(aggregate(numerator~jd,data=temp.data,sum,na.action=na.omit)[,-1]/aggregate(lambda.1~jd,data=temp.data,sum,na.action=na.omit)[,-1],
                 na.rm=T)/n.J;
  
  return(F.uncorrected - n.I/n.obs*B - n.J/n.obs*D);
};

####################
# logit Regression #
####################
PDR.logit <- function(y, model.spec, data.local){   
  n.K <- length(all.vars(model.spec))-3;
  
  model.logit <- update(model.spec, ylogit ~.);
  data.local$ylogit <- 1*(data.local[,all.vars(model.spec)[1]] <= y);
  # Logit 1(Y<=y) on X #
  lg.model <- glm(formula=model.logit, data=data.local, family=binomial(link="logit"));
  
  beta <- lg.model$coeff[1:n.K];
  se <- summary(lg.model)$coefficients[,2];
  names(se) <- paste("se.",names(lg.model$coeff),sep="");
  
  lin <- lg.model$linear.predictors;
  
  beta.corrected <- beta - bias.cor.beta(model.spec, data.local, lin, "logit");
  names(beta.corrected) <- paste("bc.",colnames(beta.corrected),sep="");

  return(c(lg.model$coeff, se, beta.corrected));
};


###########################
## Distribution Function ##
###########################
PDR.F.logit <- function(index, model.spec, data.local, ys, theta.ys, t.log2, t.d){
  y <- ys[index];
  theta <- theta.ys[index,];
  
  Y.name <- all.vars(model.spec)[1];
  
  n.obs <- NROW(data.local);
  
  w <- model.matrix(model.spec, data.local);
  lin <- w %*% as.vector(theta);
  
  
  # For log of continuous variables #
  
  F.log2.0 <- rep(sum(1*(data.local[,Y.name]<=y), na.rm=T)/n.obs, length(t.log2));
  F.log2.0.bc <- rep(0,length(t.log2));
  
  F.log2.1 <- rep(0,length(t.log2));
  F.log2.1.bc <- rep(0,length(t.log2));
  for(i in 1:length(t.log2)){
    F.log2.0.bc[i] <- bias.cor.F.logit(y, model.spec, data.local, lin, theta, "log2", t.log2[i], 0);
    
    F.log2.1[i] <- sum(plogis(lin + log(2)* theta[t.log2[i]]), na.rm=T)/n.obs;
    F.log2.1.bc[i] <- bias.cor.F.logit(y, model.spec, data.local, lin, theta, "log2", t.log2[i], 1);
  };
  names(F.log2.0)    <- paste("F.",names(theta)[t.log2],".0",sep="");
  names(F.log2.0.bc) <- paste("F.",names(theta)[t.log2],".0.bc",sep="");
  names(F.log2.1)    <- paste("F.",names(theta)[t.log2],".1",sep="");
  names(F.log2.1.bc) <- paste("F.",names(theta)[t.log2],".1.bc",sep="");
  
  
  # For binary variables #
  
  F.d.0    <- rep(0,length(t.d));
  F.d.0.bc <- rep(0,length(t.d));
  F.d.1    <- rep(0,length(t.d));
  F.d.1.bc <- rep(0,length(t.d));
  for(i in 1:length(t.d)){
    F.d.0[i]    <- sum(plogis(lin - w[,t.d[i]]*theta[t.d[i]]), na.rm=T)/n.obs;
    F.d.0.bc[i] <- bias.cor.F.logit(y, model.spec, data.local, lin, theta, "binary", t.d[i], 0);
    
    F.d.1[i]    <- sum(plogis(lin + (1-w[,t.d[i]])*theta[t.d[i]]), na.rm=T)/n.obs;
    F.d.1.bc[i] <- bias.cor.F.logit(y, model.spec, data.local, lin, theta, "binary", t.d[i], 1);
  };
  names(F.d.0)    <- paste("F.",names(theta)[t.d],".0",sep="");
  names(F.d.0.bc) <- paste("F.",names(theta)[t.d],".0.bc",sep="");
  names(F.d.1)    <- paste("F.",names(theta)[t.d],".1",sep="");
  names(F.d.1.bc) <- paste("F.",names(theta)[t.d],".1.bc",sep="");
  
  F.hat <- c(F.log2.0, F.log2.1, F.d.0, F.d.1, F.log2.0.bc, F.log2.1.bc, F.d.0.bc, F.d.1.bc);
  
  return(F.hat);
};

PDR.F.poisson <- function(index, model.spec, data.local, ys, theta.pois, lin.pois, t.log2, t.d){
  y <- ys[index];
  n.obs <- NROW(data.local);
  w <- model.matrix(model.spec, data.local);
  
  
  # For log of continuous variables #
  
  F.log2.0 <- rep(sum(ppois(y, lambda=exp(lin.pois)), na.rm=T)/n.obs, length(t.log2));
  F.log2.1 <- sapply(t.log2, function(t){sum(ppois(y, lambda=exp(lin.pois + log(2)* theta.pois[t])), na.rm=T)/n.obs});
  
  names(F.log2.0) <- paste("F.",names(theta.pois)[t.log2],".0",sep="");
  names(F.log2.1) <- paste("F.",names(theta.pois)[t.log2],".1",sep="");
  
  
  # For binary variables #
  
  F.d.0 <- sapply(t.d, function(t){sum(ppois(y, lambda=exp(lin.pois - w[,t]* theta.pois[t])), na.rm=T)/n.obs});
  F.d.1 <- sapply(t.d, function(t){sum(ppois(y, lambda=exp(lin.pois + (1-w[,t])* theta.pois[t])), na.rm=T)/n.obs});
  
  names(F.d.0) <- paste("F.",names(theta.pois)[t.d],".0",sep="");
  names(F.d.1) <- paste("F.",names(theta.pois)[t.d],".1",sep="");
  
  F.hat <- c(F.log2.0, F.log2.1, F.d.0, F.d.1);
  
  return(F.hat);
};

#########################
## Influence Functions ##
#########################
phi.logit <- function(index, model.spec, data.local, ys, theta.ys, t.log2, t.d){
  y <- ys[index];
  theta <- theta.ys[index,];
  
  w.matrix <- model.matrix(model.spec, data.local);
  n.obs <- NROW(w.matrix);
  
  Y.name <- all.vars(model.spec)[1];
  model.logit <- update(model.spec, ylogit ~. );
  data.local$ylogit <- 1*(data.local[,Y.name] <= y);
  # Logit 1(Y<=y) on X #
  lg.model <- glm(model.logit, family=binomial(link="logit"), data=data.local, na.action=na.omit);
  
    
  phi.theta <- c(1*(data.local[,Y.name]<=y) - plogis(w.matrix %*% as.vector(theta)))*w.matrix %*% vcov(lg.model)*n.obs;
  colnames(phi.theta)<- colnames(w.matrix);
  
  # For log continuous treatment variables #
  
  J.log2.0  <- replicate(length(t.log2), colSums(c(dlogis(w.matrix %*% as.vector(theta))) * w.matrix, na.rm=T)/n.obs);
  J.log2.1  <- matrix(0, nrow=NCOL(w.matrix),ncol=length(t.log2));
  for(i in 1:length(t.log2)){
    t <- t.log2[i];
    
    w.log2.1 <- w.matrix;
    w.log2.1[,t] <- log(2) + w.log2.1[,t];
    
    J.log2.1[,i]  <- colSums(c(dlogis(w.log2.1 %*% as.vector(theta))) * w.log2.1, na.rm=T)/n.obs;
    rm(w.log2.1);
  };
  
  
  # For binary treatment variables #
  
  J.d.0 <- matrix(0, nrow=NCOL(w.matrix),ncol=length(t.d));
  J.d.1 <- matrix(0, nrow=NCOL(w.matrix),ncol=length(t.d));
  for(i in 1:length(t.d)){
    t <- t.d[i];
    
    w.d.0 <- w.matrix;
    w.d.0[,t] <- 0;
    J.d.0[,i] <- colSums(c(dlogis(w.d.0 %*% as.vector(theta))) * w.d.0, na.rm=T)/n.obs;
    rm(w.d.0);
    
    w.d.1 <- w.matrix;
    w.d.1[,t] <- 1;
    J.d.1[,i] <- colSums(c(dlogis(w.d.1 %*% as.vector(theta))) * w.d.1, na.rm=T)/n.obs;
    rm(w.d.1);
  };
  
  J <- cbind(J.log2.0, J.log2.1, J.d.0, J.d.1);
  
  phi.F <- phi.theta %*% J;
  colnames(phi.F)    <- c(paste("F.",names(theta)[t.log2],".0",sep=""), paste("F.",names(theta)[t.log2],".1",sep=""),
                          paste("F.",names(theta)[t.d],".0",sep=""), paste("F.",names(theta)[t.d],".1",sep=""));
  return(cbind(phi.theta,phi.F)); # n.obs*(n.K+n.I+n.J-1 + 4);
};


########################
# Multiplier Bootstrap #
########################

# Without pairwise clustering #

boot.func <- function(b.boot, phi, variance){
  # phi is a N*P*n.thres matrix, containing influence functions for P estimates.
  n.obs <- NROW(phi);
  
  # Generate weights #
  set.seed(b.boot);
  w.bt <- rnorm(n.obs);
  w.b <- w.bt - mean(w.bt);
  rm(w.bt);
  
  delta <- colMeans((phi*w.b)); # P*n.thres or n.ae*1
  
  #variance <- apply((phi*w.b)^2, dimension, mean)/n.obs;
  delta[variance==0] <- 0;
  
  if(length(dim(phi))==2){ # for Average Effects;
    t.stat <- abs(delta)/sqrt(variance);
    result <- rbind(delta,t.stat);
  }else{
    t.stat <- apply(abs(delta)/sqrt(variance),1,max,na.rm=T);
    result <- t(cbind(delta, t.stat));
  }
  
  # Return a (n.thres+1)*P of deltas and t.stat #
  return(result); 
};


# With pairwise clustering #

boot.func.pwclus <- function(b.boot, model.spec, data.local, phi, variance){
  # phi is a N*P*n.thres matrix, containing influence functions for P estimates.
  # variance is the pairwise clustering adjusted.
  n.obs <- NROW(phi);
  
  i.name <- all.vars(model.spec)[n.K+2];
  j.name <- all.vars(model.spec)[n.K+3];

  id <- data.local[,i.name];
  jd <- data.local[,j.name];
  
  # Generate weights #
  set.seed(b.boot);
  w.bt <- rep(0,n.obs);
  w.bt[id<=jd] <- rnorm(sum(id<=jd));
  w.bt.reversed <- w.bt[id<jd];
  w.bt[id >jd] <- w.bt.reversed[order(jd[id<jd], id[id<jd])];
  w.b <- w.bt - mean(w.bt);
  rm(w.bt, w.bt.reversed);

  
  delta <- colMeans((phi*w.b)); # P*n.thres or n.thres*1
  
  #phi.reversed <- phi[order(jd,id),,];
  #variance <- apply((phi+phi.reversed)*phi, c(2,3), mean)/n.obs;
  #rm(phi.reversed);
  delta[variance==0] <- 0;
  
  if(length(dim(phi))==2){
    t.stat <- abs(delta)/sqrt(variance);
    result <- rbind(delta,t.stat);
  }else{
    t.stat <- apply(abs(delta)/sqrt(variance),1,max,na.rm=T);
    result <- t(cbind(delta, t.stat));
  }
  
  # Return a (n.thres+1)*P of deltas and t.stat #
  return(result); 
};




#########################
# Montecarlo Simulation #
#########################
dgp <- function(wu, theta.0, ys){
  w <- head(wu,-1);
  u <- tail(wu,1);
  
  lin.index <- (theta.0 %*% as.vector(w) <= u);
  if(any(lin.index)){
    Y <- ys[max(which(lin.index),na.rm=T)];
  }else{
    Y <- 0;
  }
  
  return(Y);
};