####################################################################;
# TRADE APPLICATION:  STATIC LOGIT MODEL; 
####################################################################;

# Empirical application of the paper "Panel Quantile Effects Via Distribution Regression"
# Authors: V. Chernozhukov, I. Fernandez-Val, M. Weidner

# Source: Helpman, Melitz, Rubinstein: "Estimating Trade Flows: Trading Partners and Trading Volumes, 2008,
# QJE, 123(2), pp. 441-487;

# Data: Helpman and Melitz web site;

# Std errors assume correct specification;

# 100% increase in distance;

# Source directory in SCC "/project/ivangrp/PDR/";

rm(list=ls());

library(foreign);
library(parallel);
library(Matrix);
library(MASS);
library(speedglm);


####################
# Parallel Setting #
####################

# Calculate the namber of cores
n.cores <- as.numeric(Sys.getenv("NSLOTS"));
if(is.na(n.cores))n.cores<-1;


source("PDR_Source.R");


################
# Data Loading #
################
trade.data <- read.dta("trade.dta");
trade.data <- trade.data[!is.na(trade.data$trade),];

trim <- 100;
str.trim <- ifelse(trim==100, "",paste("_c",trim,sep=""));
trade.data <- subset(trade.data, vtrade<=quantile(vtrade,trim/100, na.rm=T));

model.spec <- vtrade ~ 0 + ldist + legal + border + language + colony + currency + fta + religion + factor(id) + factor(jd);
n.K <- length(all.vars(model.spec))-3; # number of explanatory variables --  without dependent and two fixed effect variables;
# Treatment variables of which counterfactuals are estimated: #
t.log2 <- 1; #ldist;
t.d <- 2; #legal; It can be c(2,5), e.g.;

Y.name <- all.vars(model.spec)[1];
X.name <- all.vars(model.spec)[2:(n.K+1)];

ys   <- quantile(trade.data[,Y.name],c(0:200)/200, na.rm=T);
taus <- c(0:200)[!duplicated(ys,fromLast=T)]/200;
ys   <- ys[!duplicated(ys,fromLast=T)];
n.thres <- length(ys);


e.beta.l <- mclapply(as.list(ys),PDR.logit, mc.cores=n.cores, model.spec=model.spec, data.local=trade.data);
e.beta   <- do.call(rbind,e.beta.l);
rm(e.beta.l);



#library(pryr);
#print(mem_used());

theta.ys <- e.beta[,1:((NCOL(e.beta)-n.K)/2)];
e.F.l <- mclapply(1:n.thres, PDR.F.logit, mc.cores=n.cores, model.spec=model.spec, data.local=trade.data, ys=ys, 
                  theta.ys=theta.ys, t.log2=t.log2, t.d=t.d); # t.log2/t.d is the index of log continuos/binary treatment(the place in model.spec);
e.F <- do.call(rbind, e.F.l); # n.thres*8;
rm(e.F.l);

save.image(paste("EmpApp_Est",str.trim,".RData", sep=""));
